using System;
using System.Collections;
using System.Reflection;
using System.Text;
using Microsoft.SqlServer.Dts.Runtime;
using Microsoft.SqlServer.Dts.Runtime.Wrapper;
using Microsoft.SqlServer.Dts.Pipeline;
using Microsoft.SqlServer.Dts.Pipeline.Wrapper;

// A transformation for profiling data.
// The current set of profiles are limited, but can easily be
// augmented by adding new profile type classes.
namespace SSIS2K5.Samples.Transforms.DataProfiler
{
    #region Column Profiler classes
    class ColumnProfilerBase
    {
        public int ColumnIndex;
        public string ColumnName;
        public DTP_BUFFCOL ColumnInfo;

        public virtual string GetProfileType() { return ""; }
        public virtual string GetProfileValue() { return ""; }

        public virtual bool CanProfile() { return true; }
        public virtual void Prepare() { }
        public virtual void Profile(PipelineBuffer buffer) { }
    };
    // A way to convert types into strings.
    public abstract class ValueProfiler
    {
        public abstract void Profile(object o);
        public abstract string GetValue();
    };
    #endregion

    #region PipelineComponent
    [DtsPipelineComponent(
		DisplayName = "Data Profiler",
		Description = "Computes profile metrics for input data",
		IconResource = "SSIS2K5.Samples.Transforms.DataProfiler.Component.ico",
		ComponentType = ComponentType.Transform )]
    public class DataProfiler : PipelineComponent
    {    
        #region Privates
        // Locations of columns of the buffer. These need to be computed in
        // PreExecute because they depend on particulars of the buffer layout,
        // which can differ from package to package.
        private int columnIndex;
        private int typeIndex;
        private int profileTypeIndex;
        private int profileValueIndex;
        
        // Output buffer. Needs to be member so we can 
        // access it whenever we wish to add a row to the buffer
        private PipelineBuffer outputBuffer;

        // Column profilers.
        private ArrayList columnProfilers = new ArrayList();
        private const int columnProfilerCount = 4;
        #endregion
        #region Overrides
        public override void ProvideComponentProperties()
		{
			base.ProvideComponentProperties();

            ComponentMetaData.InputCollection[0].Name = "Data Profiler Input";

            IDTSOutput90 output = ComponentMetaData.OutputCollection[0];
            output.Name = "Data Profiler Output";

            // Specify that this is an Asynchronous output
            output.SynchronousInputID = 0;

            IDTSOutputColumnCollection90 outputColumnCollection =
                output.OutputColumnCollection;

            IDTSOutputColumn90 outputColumn = outputColumnCollection.New();
            outputColumn.Name = "Column";
            outputColumn.SetDataTypeProperties(
                DataType.DT_WSTR, 50, 0, 0, 0);

            outputColumn = outputColumnCollection.New();
            outputColumn.Name = "Type";
            outputColumn.SetDataTypeProperties(
                DataType.DT_WSTR, 50, 0, 0, 0);

            outputColumn = outputColumnCollection.New();
            outputColumn.Name = "ProfileType";
            outputColumn.SetDataTypeProperties(
                DataType.DT_WSTR, 10, 0, 0, 0);

            outputColumn = outputColumnCollection.New();
            outputColumn.Name = "ProfileValue";
            outputColumn.SetDataTypeProperties(
                DataType.DT_WSTR, 50, 0, 0, 0);
        }

		public override DTSValidationStatus Validate()
		{
			return DTSValidationStatus.VS_ISVALID;
		}

      public override void PreExecute()
        {
            base.PreExecute();

            #region Determine locations of output columns in output buffer

            IDTSOutput90 output = ComponentMetaData.OutputCollection[0];

            IDTSOutputColumnCollection90 outputColumnCollection =
                output.OutputColumnCollection;

            columnIndex = BufferManager.FindColumnByLineageID(
                output.Buffer, outputColumnCollection["Column"].LineageID);

            typeIndex = BufferManager.FindColumnByLineageID(
                output.Buffer, outputColumnCollection["Type"].LineageID);

            profileTypeIndex = BufferManager.FindColumnByLineageID(
                output.Buffer, outputColumnCollection["ProfileType"].LineageID);

            profileValueIndex = BufferManager.FindColumnByLineageID(
                output.Buffer, outputColumnCollection["ProfileValue"].LineageID);

            #endregion

            #region Create an array of column profiler objects

            IDTSInput90 input = ComponentMetaData.InputCollection[0];

            IDTSInputColumnCollection90 inputColumnCollection =
                input.InputColumnCollection;

            columnProfilers.Capacity = inputColumnCollection.Count * columnProfilerCount;

            // For each input column...
            foreach(IDTSInputColumn90 inputColumn in inputColumnCollection)
            {
                // For each column profiler...
                for (int j = 0; j < columnProfilerCount; j++)
                {
                    ColumnProfilerBase columnProfiler = null;

                    // Create column profiler object of the desired type
                    switch(j)
                    {
                        case 0:
                            columnProfiler = new ColumnProfilerMin();
                            break;

                        case 1:
                            columnProfiler = new ColumnProfilerMax();
                            break;

                        case 2:
                           columnProfiler = new ColumnProfilerAvgStrLen();
                            break;

                        case 3:
                            columnProfiler = new ColumnProfilerPercentageNulls();
                            break;
                    }

                    // Give column profiler the information about the input column
                    columnProfiler.ColumnIndex = BufferManager.FindColumnByLineageID(
                        input.Buffer,
                        inputColumn.LineageID);

                    columnProfiler.ColumnName = inputColumn.Name;

                    // And tell the details of that column (type, length, etc)
                    BufferManager.GetColumnInfo(
                        input.Buffer,
                        columnProfiler.ColumnIndex,
                        ref columnProfiler.ColumnInfo);

                    // The column profiler might not be able to profile this column,
                    // so ask him beforehand.
                    if (!columnProfiler.CanProfile())
                    {
                        // If we can't profile this column, null out the column profiler
                        // object and our ProcessInput method will ignore this element.
                        columnProfiler = null;
                    }
                    else
                    {
                        columnProfiler.Prepare();
                    }

                    // Save the new object in the array of column profilers
                    columnProfilers.Add(columnProfiler);
                }
            }

            #endregion
        }

      public override void PostExecute()
        {
 	        base.PostExecute();
            columnProfilers.Clear();
        }

      public override void PrimeOutput(int outputs, int[] outputIDs, PipelineBuffer[] buffers)
        {
            outputBuffer = buffers[0];
        }

		public override void ProcessInput(int inputID, PipelineBuffer buffer)
		{
            // Check if this is all the data
            if (!buffer.EndOfRowset)
            {
                // While there are rows, profile columns.
                while (buffer.NextRow())
                {
                    // Iterate through the profilers and apply the profiles
                    for (int i = 0; i < columnProfilers.Count; i++)
                    {
                        ColumnProfilerBase columnProfiler =
                            (ColumnProfilerBase)columnProfilers[i];

                        if (columnProfiler != null)
                        {
                            columnProfiler.Profile(buffer);
                        }
                    }
                }
            }
            else
            {
                // When we reach end of rowset, output final profile values.
                for (int i = 0; i < columnProfilers.Count; i++)
                {
                    ColumnProfilerBase columnProfiler = 
                        (ColumnProfilerBase) columnProfilers[i];

                    if (columnProfiler != null)
                    {
                        outputBuffer.AddRow();

                        outputBuffer[columnIndex] = columnProfiler.ColumnName;
                        outputBuffer[typeIndex] = columnProfiler.ColumnInfo.DataType.ToString();
                        outputBuffer[profileTypeIndex] = columnProfiler.GetProfileType();
                        outputBuffer[profileValueIndex] = columnProfiler.GetProfileValue();
                    }
                }

                outputBuffer.SetEndOfRowset();
                outputBuffer = null;
            }
        }
        #endregion
    }
    #endregion
}
